/*
 *  Time-based histogram maker.
 *  Based on the mipHistogram_maker.C code.  Modified by Wayne Witzke, for the
 *  University of Kentucky, Department of Physics & Astronomy.
 */

#include <iostream>
#include <fstream>
#include <set>
#include <climits>
#include <map>
using namespace std;

void checkTowers(const char *file_list="/star/u/wwitzke/workspace/mipcode/data/FileLists/2008_file_list_partial",
			 //const char *file_list="/star/data07/EMC/mattheww/offline_tower_calibration/2006/ppProd/root/calib/BE6089404D8E758EE247FC81E96086D9_*.root",
			 //const char* file_list="/star/data07/EMC/mattheww/offline_tower_calibration/2006/ppProd/root/mip/*mip.root",
			 const char* skimfile="/star/institutions/uky/wwitzke/mip/mipskimfile.root") 

{
    gROOT->Macro("LoadLogger.C");
    gROOT->Macro("$STAR/StRoot/StMuDSTMaker/COMMON/macros/loadSharedLibraries.C");
    gSystem->Load("StTpcDb");
    gSystem->Load("StDaqLib");
    gSystem->Load("StDetectorDbMaker");
    gSystem->Load("St_db_Maker");
    gSystem->Load("StDbUtilities");
    gSystem->Load("StEmcRawMaker");
    gSystem->Load("StMcEvent");
    gSystem->Load("StMcEventMaker");//***
    gSystem->Load("StEmcSimulatorMaker");//***
    gSystem->Load("StEmcADCtoEMaker");
    gSystem->Load("StEpcMaker");
    gSystem->Load("StDbBroker");
    gSystem->Load("StEEmcUtil");
    gSystem->Load("StAssociationMaker");
    gSystem->Load("StEmcTriggerMaker");

    gSystem->Load("StEmcOfflineCalibrationMaker");

    const int ntowers=4800;
    const int neta = 40;
    const bool lookForSwaps=false;

    cout<<"input filelist:  "<<file_list<<endl;
    cout<<"histogram file:  "<<skimfile<<endl;

    //chain all input files together
    char file[60000];
    TChain* calib_tree = new TChain("calibTree");
    ifstream filelist(file_list);
    TFile *test_file;
    /*
    while(1)
    {
	filelist >> file;
	cout<<" File="<<file<<endl;
	if(!filelist.good()) break;
	calib_tree->Add(file);
    }
    */

    StEmcOfflineCalibrationEvent* myEvent = new StEmcOfflineCalibrationEvent();
    calib_tree->SetBranchAddress("event_branch",&myEvent);
    StEmcOfflineCalibrationTrack* mip;
    StEmcGeom etaFinder("bemc");

    int curModule_J;
    int curEta_J;
    int curSub_J;
    /*
    float curEta;
    if ( etaFinder.getEta( myID, curEta ) )
    {
	cout << "Eta error. . ?" << endl;
	cout.flush();
	continue;
    }
    */
    //cout << "Eta is: " << curEta << endl;
    //cout.flush();
    for ( unsigned int ii = 0; ii < 4801; ++ii )
    {
	if (
	    etaFinder.getBin(
		ii, curModule_J, curEta_J, curSub_J
	    )
	)
	{
	    cout << "Eta bin error. . ?" << endl;
	    cout.flush();
	    continue;
	}
	cout << ii << " : ";
	cout << "Module is: " << curModule_J << " : ";
	cout << "Eta is: " << curEta_J << " : ";
	cout << "Sub is: " << curSub_J << endl;
    }

    return;

    //create the 4800 mip histograms
    /*
    char name[100];
    TH1* mipHisto[ntowers];
    for(int k=0; k<ntowers; k++)
    {
	sprintf(name,"mipHisto_%i",k+1);
	mipHisto[k] = new TH2D(name,name,250,-50.5,199.5);
    }
    */

    //	It turns out we have to preprocess all entries so that we know the
    //	range of run numbers.
    unsigned int minFill = numeric_limits<unsigned int>::max();
    unsigned int maxFill = 0;
    unsigned int nentries = calib_tree->GetEntries();
    cout << "Preprocessing to find run numbers. . ." << endl;
    cout << "(total entries " << nentries << ")" << endl;

    //	We're pulling the run and fill information from a file that has been
    //	preprocessed.  We're going to put them into a map so that we can look
    //	up fills by run number.
    map<int, int> fills;
    int curRun, curFill;
    ifstream runFillsFile( "/star/u/wwitzke/workspace/mipcode/data/PreProcessing/runNumbers.csv" );

    while ( runFillsFile >> curRun >> curFill )
    {
	fills.insert( pair<int, int>( curRun, curFill ) );
	if ( curFill < minFill )
	    minFill = curFill;

	if ( curFill > maxFill )
	    maxFill = curFill;
    }
    runFillsFile.close();

    cout << "Minimum fill number: " << minFill << endl;
    cout << "Maximum fill number: " << maxFill << endl;

    /*
    char * name = "time_mipHisto";
    //	I want to create a two-dimensionsal bin here, so that I can see the
    //	variations of bins over time.  
    TH2D * mipHisto = new TH2D(name,name,250,-50.5,199.5,runs.size(),minRun,maxRun);
    //mipHisto->SetBit(TH1::kCanRebin);
    */

    //	We are going to create a separate histogram for each fill number.
    char name[100];
    TH1*** mipHisto = new TH1**[maxFill-minFill + 1];
    for(unsigned int ii = 0; ii < maxFill-minFill+1; ++ii)
    {
	mipHisto[ii] = new TH1*[neta];
	for (unsigned int jj = 0; jj < neta; ++jj)
	{
	    sprintf(name,"mipHisto_run_%i_eta_%i",minFill+ii,jj+1);
	    mipHisto[ii][jj] = new TH1D(name,name,250,-50.5,199.5);
	}
    }

    //keep track of all hit towers and exclude any with >1 track/tower
    set<int> track_towers;
    set<int> excluded_towers;

    int badADC = 0;
    int bigNeg = 0;
    int zeroADCTower[4801];
    int allTower[4801];
    for ( unsigned int ii = 0; ii < 4801; ++ii )
    {
	zeroADCTower[ii] = 0;
	allTower[ii] = 0;
    }

    for(unsigned int ii=0; ii<nentries; ++ii)
    {
	calib_tree->GetEntry(ii);

	unsigned int runNum = myEvent->run;

	if(ii%100000 == 0)
	{
	    cout << "processing " << ii << " of " << nentries << endl;
	    cout << "Processing run number: " << runNum << endl;
	}

	int trackEntries = myEvent->tracks->GetEntries();

	if ( !trackEntries )
	    continue;

	//we're only working with events from vertexIndex==0
	if(TMath::Abs(myEvent->vz[0]) > 30.)
	    continue;

	//  We can also grab the run number here.  We'll be using this to
	//  speicfy the y-bin to fill later on.  This is more efficient than
	//  dereferencing the run number every time we do a fill.

	track_towers.clear();
	excluded_towers.clear();

	//do a quick loop over tracks to get excluded towers
	for ( unsigned int jj = 0; jj < trackEntries; ++jj )
	{
	    mip = (StEmcOfflineCalibrationTrack*)myEvent->tracks->At(jj);
	    int id = mip->tower_id[0];

	    if(track_towers.find(id) != track_towers.end()){
	    excluded_towers.insert(id);
	    }
	    else
	    {
		track_towers.insert(id);
	    }
	}   

	//select on runnumbers to look for stability
	//if(myEvent->run < 7135000) continue;

	for ( unsigned int jj = 0; jj < trackEntries; ++jj )
	{
	    mip = (StEmcOfflineCalibrationTrack*)myEvent->tracks->At(jj);

	    int myID = mip->tower_id[0];

	    if ( mip->tower_adc[0] == 0 )
	    {
		++zeroADCTower[myID];
	    }
	    ++allTower[myID];

	    //subtract pedestal
	    double pedsub = mip->tower_adc[0] - mip->tower_pedestal[0];

	    if(mip->p < 1.)
		continue;
	    if(mip->tower_status[0] != 1)
		continue;
	    if(mip->highest_neighbor > 2.)
		continue;
	    if(TMath::Abs(pedsub) < 1.5*mip->tower_pedestal_rms[0])
		continue;
	    if(myID != mip->tower_id_exit)
		continue;
	    if(mip->vertexIndex > 0)
		continue;
	    if(excluded_towers.find(myID) != excluded_towers.end())
		continue;
		

	    //	This is how it *should* be working, but unfortunately we're
	    //	using old code.
	    //int curEta = (int) mip->eta*40;
	    //	So, instead, to get eta, we use this:
	    int curModule_I;
	    int curEta_I;
	    int curSub_I;
	    /*
	    float curEta;
	    if ( etaFinder.getEta( myID, curEta ) )
	    {
		cout << "Eta error. . ?" << endl;
		cout.flush();
		continue;
	    }
	    */
	    //cout << "Eta is: " << curEta << endl;
	    //cout.flush();
	    if (
		etaFinder.getBin(
		    myID, curModule_I, curEta_I, curSub_I
		)
	    )
	    {
		cout << "Eta bin error. . ?" << endl;
		cout.flush();
		continue;
	    }
	    //cout << "Integer eta is: " << curEta_I << endl;
	    //cout.flush();

	    if ( curEta_I < 0 || curEta_I > 39 )
	    {
		cout << curEta_I << "!" << endl;
		cout.flush();
	    }

	    if(pedsub < 0)
	    {
		cout << "-------------------------------------" << endl;
		cout << "pedsub LESS THAN 0!" << endl;
		cout << "Tower ID: " << myID << " (ADC=" << mip->tower_adc[0] << ")" << endl;
		cout << "Tower Status: " << mip->tower_status[0] << endl;
		cout << "Eta: " << curEta_I << endl;
		cout << "Run: " << runNum << endl;
		cout << "Fill: " << fills[runNum] << endl;
		cout << "Tower ADC: " << mip->tower_adc[0] << endl;
		cout << "Tower Pedistal: " << mip->tower_pedestal[0] << endl;
		cout << "pedsub: " << pedsub << endl;
		cout << "Pedistal RMS: " << mip->tower_pedestal_rms[0] << endl;
		if ( mip->tower_adc[0] == 0 )
		{
		    ++badADC;
		    //++zeroADCTower[myID];
		}
		if ( pedsub < -5 )
		{
		    ++bigNeg;
		}
	    }
	    /*
	    ++zeroADCTower[myID];
	    ++allTower[myID];
	    */

	    mipHisto[fills[runNum]-minFill][curEta_I]->Fill(pedsub);
	}
    }

    cout << "-------------------------------------------------" << endl;
    cout << "Number of 0 ADC: " << badADC << endl;
    cout << "Number of pedsub < -5: " << bigNeg << endl;

    cout << "Zero-only towers" << endl;
    for ( unsigned int ii = 0; ii < 4801; ++ii )
    {
	if ( zeroADCTower[ii] != 0 )
	{
	    cout << "Tower " << ii << " had " << zeroADCTower[ii] << " zero's out of " << allTower[ii] << endl;;
	}
    }
    cout << endl << endl << "ALL towers" << endl;
    for ( unsigned int ii = 0; ii < 4801; ++ii )
    {
	cout << "Tower " << ii << " had " << zeroADCTower[ii] << " zero's out of " << allTower[ii] << endl;;
    }

    TFile* output_file = new TFile(skimfile,"RECREATE");
    for ( int kk=0; kk < maxFill-minFill+1; ++kk )
	for ( int ll = 0; ll < neta; ++ll )
	    if ( mipHisto[kk][ll]->GetEntries() > 0 )
		mipHisto[kk][ll]->Write();
    //mipHisto->Write();
    output_file->Close();
    cout << "Done writing histograms" << endl;
    cout.flush();
}
