#include <iostream>
#include <fstream>
#include <cmath>
using namespace std;

//#include "TPostScript.h"
#include "TCanvas.h"
#include "TStyle.h"
#include "TGraphErrors.h"

/*  The point of this function is to save individual graphs and graph composits
 *  as png images.
 */

void gainsOverTimeImages(
    const char* file="mip2008.gains.corrected",
    const char* savePath="/star/institutions/uky/wwitzke/images/2008/"
)
{
    const int rebinNumber = 4;

    //	This is for record keeping in the output files.
    cout << "input file:  " << file << endl;
    cout << "save path: " << savePath << endl;

    //	Setting up the. . ?
    /*
    gStyle->SetCanvasColor( 10 );
    gStyle->SetCanvasBorderMode( 0 );
    gStyle->SetStatColor( 10 );
    gStyle->SetOptTitle( 0 );
    gStyle->SetOptFit( 111 );
    gStyle->SetOptStat( "oume" );
    */

    //TPostScript *ps = new TPostScript( postscript );
    //TCanvas *myCanvas = new TCanvas( "c", "", 100, 100, 600., 800. );
    TCanvas *myCanvas;
    int pad;

    //	This is our input file.
    ifstream infile( file );

    const int neta = 40;
    TGraphErrors *myGraph[neta];

    for ( int ii = 0; ii < neta; ++ii )
    {
	myGraph[ ii ] = new TGraphErrors();
    }

    TGraphErrors *myMeans[neta];

    for ( int ii = 0; ii < neta; ++ii )
    {
	myMeans[ ii ] = new TGraphErrors();
    }

    TGraphErrors *myFittable[neta];

    for ( int ii = 0; ii < neta; ++ii )
    {
	myFittable[ ii ] = new TGraphErrors();
    }

    int counters[neta];

    int specCounters[neta];

    char graphName[200];
    int fill = -1;
    int oldFill = -1;
    int eta;
    double fillIncrement = 0;
    double gain, width, mean, meanError;

    //	We'll create the graphs first, and draw them later.
    while ( infile >> fill >> eta >> gain >> width >> mean >> meanError )
    {
	//  So, fillIncrement here relies on the fill numbers in the file being
	//  grouped together. . .  This is a bad assumption to make in general,
	//  and while it will work for the files that I have put together, I
	//  really hate that this is not more generalized.  I highly recommend
	//  that someone take the time to make this more generalized.
	if ( oldFill != fill )
	{
	    ++fillIncrement;
	    oldFill = fill;
	}
	++counters[ eta - 1 ];

	myGraph[ eta - 1 ]->Set( counters[ eta - 1 ] );
	myGraph[ eta - 1 ]->SetPoint(
	    counters[ eta - 1 ], (double) fillIncrement, gain
	);
	myGraph[ eta - 1 ]->SetPointError( counters[ eta - 1 ], 0, width );
	myMeans[ eta - 1 ]->Set( counters[ eta - 1 ] );
	myMeans[ eta - 1 ]->SetPoint(
	    counters[ eta - 1 ], (double) fillIncrement, mean
	);
	myMeans[ eta - 1 ]->SetPointError( counters[ eta - 1 ], 0, meanError );
	if ( gain != 0 )
	{
	    myFittable[ eta - 1 ]->Set( specCounters[ eta - 1 ] );
	    myFittable[ eta - 1 ]->SetPoint(
		specCounters[ eta - 1 ], (double) fillIncrement, gain
	    );
	    myFittable[ eta - 1 ]->SetPointError(
		specCounters[ eta - 1 ], 0, width
	    );
	    ++specCounters[ eta - 1 ];
	}
    }

    infile.close();

    TF1 * tempFitter;
    char fitname[200];
    char filename[400];
    double chi2;
    TGraph * nbinsGraph = new TGraph( 40 );

    double ymax = 0;

    for ( int ii = 0; ii < neta; ++ii )
    {
	sprintf( fitname, "Fit for eta %i", ii+1 );
	tempFitter = new TF1( fitname, "pol0", 1., (double) fillIncrement );
	//tempFitter = new TF1( fitname, "pol0", 1., 25. );
	//tempFitter = new TF1( fitname, "pol0", 30., (double) fillIncrement );

	myFittable[ ii ]->Fit( tempFitter, "rq" );
	chi2 = tempFitter->GetChisquare() / ((double) fillIncrement - 1);
	//chi2 = tempFitter->GetChisquare() / 25.;
	//chi2 = tempFitter->GetChisquare() / ((double) fillIncrement - 30.);

	if ( chi2 > ymax )
	{
	    ymax = chi2;
	}

	nbinsGraph->Set( ii );
	nbinsGraph->SetPoint( ii, ii + 1, chi2 );

	/*
	if ( ii % 1 == 0 )
	{
	    myCanvas->Update();
	    ps->NewPage();
	    myCanvas->Clear();
	    myCanvas->Divide( 1, 2 );
	    pad = 1;
	}

	myCanvas->cd( pad );
	*/
	myCanvas = new TCanvas( "c", "", 1);

	sprintf ( graphName, "Eta Bin %i mip Positions Vs. Time", ii+1 );

	//  Beautifing the graph.
	myMeans[ ii ]->GetYaxis()->SetRangeUser( 12, 25 );
	myMeans[ ii ]->GetXaxis()->SetRangeUser( 0, fillIncrement+1 );
	myMeans[ ii ]->SetMarkerStyle( 26 );
	myMeans[ ii ]->SetMarkerSize( .5 );
	myMeans[ ii ]->SetMarkerColor( 3 );

	myGraph[ ii ]->SetTitle( graphName );
	myGraph[ ii ]->GetXaxis()->SetTitle( "\"Fill Time\"" );
	myGraph[ ii ]->GetYaxis()->SetTitle( "mip Position" );
	myGraph[ ii ]->GetYaxis()->SetRangeUser( 12, 25 );
	myGraph[ ii ]->GetXaxis()->SetRangeUser( 0, fillIncrement+1 );
	myGraph[ ii ]->SetMarkerStyle( 25 );
	myGraph[ ii ]->SetMarkerSize( .5 );
	myGraph[ ii ]->SetMarkerColor( 4 );

	tempFitter->SetTitle( graphName );
	tempFitter->GetYaxis()->SetRangeUser( 12, 25 );
	tempFitter->GetXaxis()->SetRangeUser( 0, fillIncrement+1 );
	tempFitter->SetLineWidth( 2 );
	tempFitter->SetLineColor( 2 );
	tempFitter->SetLineStyle( 1 );

	//  Draw the graph.
	myGraph[ ii ]->Draw( "AP" );
	myMeans[ ii ]->Draw( "PSAME" );
	tempFitter->Draw( "LSAME" );

	//++pad;

	sprintf(filename, "%sfit_eta_%i_main.png", savePath, ii+1 );
	myCanvas->SaveAs(filename);
	delete myCanvas;

	//delete tempFitter;
    }

    myCanvas = new TCanvas( "c", "", 1);
    /*
    myCanvas->Update();
    ps->NewPage();
    myCanvas->Clear();
    myCanvas->Divide( 1, 2 );
    myCanvas->cd( 1 );
    */

    sprintf( graphName, "Chi Squared vs. Bin Number" );

    nbinsGraph->SetTitle( graphName );
    nbinsGraph->GetXaxis()->SetTitle( "Bin Number" );
    nbinsGraph->GetYaxis()->SetTitle( "Chi^2/NDF" );
    nbinsGraph->SetMarkerStyle( 5 );
    nbinsGraph->SetMarkerSize( .5 );
    nbinsGraph->SetMarkerColor( 1 );

    //	THIS SHOULD FREAKING WORK!
    //double xmin, ymin, xmax, ymax;
    //nbinsGraph->ComputeRange( xmin, ymin, xmax, ymax );

    if ( ymax > 10 )
    {
	ymax /= 10;
	ymax = std::ceil( ymax );
	ymax *= 10;
    }
    else
    {
	ymax = std::ceil( ymax );
    }

    nbinsGraph->GetYaxis()->SetRangeUser( 0, ymax );

    nbinsGraph->Draw( "AP" );

    sprintf(filename, "%schisquared_main.png", savePath, ii+1 );
    myCanvas->SaveAs(filename);
    delete myCanvas;

    TGraphErrors *tempGraph;
    nbinsGraph = new TGraph();
    pad = 2;

    for ( int ii = 0; ii < neta; ++ii )
    {
	tempGraph = rebin( myFittable[ ii ], rebinNumber );

	sprintf(
	    fitname, "Fit for eta %i (rebinning by %i)", ii+1, rebinNumber
	);
	tempFitter =
	    new TF1( fitname, "pol0", 1., (double) (fillIncrement/rebinNumber) );
	//tempFitter = new TF1( fitname, "pol0", 1., 25. );
	//tempFitter = new TF1( fitname, "pol0", 30., (double) fillIncrement );

	tempGraph->Fit( tempFitter, "rq" );
	chi2 =
	    tempFitter->GetChisquare() / ((double) (fillIncrement/rebinNumber)-1);
	//chi2 = tempFitter->GetChisquare() / 25.;
	//chi2 = tempFitter->GetChisquare() / ((double) fillIncrement - 30.);

	if ( chi2 > ymax )
	{
	    ymax = chi2;
	}

	nbinsGraph->Set( ii );
	nbinsGraph->SetPoint( ii, ii + 1, chi2 );

	myCanvas = new TCanvas( "c", "", 1);

	/*
	if ( ii % 1 == 0 )
	{
	    myCanvas->Update();
	    ps->NewPage();
	    myCanvas->Clear();
	    myCanvas->Divide( 1, 2 );
	    pad = 1;
	}

	myCanvas->cd( pad );
	*/

	sprintf (
	    graphName, "Eta Bin %i mip Positions Vs. Time (rebinned by %i)",
	    ii+1, rebinNumber
	);

	//  Beautifing the graph.
	tempGraph->SetTitle( graphName );
	tempGraph->GetXaxis()->SetTitle( "\"Fill Time\"" );
	tempGraph->GetYaxis()->SetTitle( "mip Position" );
	tempGraph->GetYaxis()->SetRangeUser( 12, 25 );
	tempGraph->GetXaxis()->SetRangeUser(
	    0, (fillIncrement/rebinNumber) + 1
	);
	tempGraph->SetMarkerStyle( 25 );
	tempGraph->SetMarkerSize( .5 );
	tempGraph->SetMarkerColor( 4 );

	tempFitter->SetTitle( graphName );
	tempFitter->GetYaxis()->SetRangeUser( 12, 25 );
	tempFitter->GetXaxis()->SetRangeUser(
	    0, (fillIncrement/rebinNumber) + 1
	);
	tempFitter->SetLineWidth( 2 );
	tempFitter->SetLineColor( 2 );
	tempFitter->SetLineStyle( 1 );

	//  Draw the graph.
	tempGraph->Draw( "AP" );
	tempFitter->Draw( "LSAME" );

	sprintf(filename, "%sfit_eta_%i_rebin.png", savePath, ii+1 );
	myCanvas->SaveAs(filename);
	delete myCanvas;

	//++pad;

	//delete tempFitter;
    }

    myCanvas = new TCanvas( "c", "", 1);
    /*
    myCanvas->Update();
    ps->NewPage();
    myCanvas->Clear();
    myCanvas->Divide( 1, 2 );
    myCanvas->cd( 1 );
    */

    sprintf( graphName, "Chi Squared vs. Bin Number" );

    nbinsGraph->SetTitle( graphName );
    nbinsGraph->GetXaxis()->SetTitle( "Bin Number" );
    nbinsGraph->GetYaxis()->SetTitle( "Chi^2/NDF" );
    nbinsGraph->SetMarkerStyle( 5 );
    nbinsGraph->SetMarkerSize( .5 );
    nbinsGraph->SetMarkerColor( 1 );

    //	THIS SHOULD FREAKING WORK!
    //double xmin, ymin, xmax, ymax;
    //nbinsGraph->ComputeRange( xmin, ymin, xmax, ymax );

    if ( ymax > 10 )
    {
	ymax /= 10;
	ymax = std::ceil( ymax );
	ymax *= 10;
    }
    else
    {
	ymax = std::ceil( ymax );
    }

    nbinsGraph->GetYaxis()->SetRangeUser( 0, ymax );

    nbinsGraph->Draw( "AP" );

    sprintf(filename, "%schisquared_rebin.png", savePath, ii+1 );
    myCanvas->SaveAs(filename);
    delete myCanvas;

    //ps->Close();

    cout << "Done" << endl; 
}

//  Here we "rebin" the TGraph that gets passed in so points are combined,
//  which should, hopefully, result in easier-to-detect larger patterns.
TGraphErrors * rebin( TGraphErrors * inGraph, int rebinNumber )
{
    TGraphErrors * retVal = new TGraphErrors();

    double x, y, sigma;
    double sumy, sumSigmaSquared;
    int outCounter = 0;

    //	So, earlier we deliberately filtered points with a y-value of zero, and
    //	that means that, here, it should be safe to test y to see if it is
    //	zero, and skip the point if it is zero.
    int pointNum = inGraph->GetMaxSize();
    for ( int ii = 0; ii < pointNum; ++ii )
    {
	inGraph->GetPoint( ii, x, y );
	if ( y == 0. )
	{
	    cout << "Got here!" << endl;
	    continue;
	}

	if ( (ii % rebinNumber) == 0 )
	{
	    if ( sumSigmaSquared != 0 )
	    {
		retVal->Set( outCounter );
		retVal->SetPoint(
		    outCounter, outCounter, sumy/sumSigmaSquared
		);
		retVal->SetPointError(
		    outCounter, 0, std::sqrt(1/sumSigmaSquared)
		);
	    }
	    ++outCounter;
	    sumy = sumSigmaSquared = 0;
	}

	sigma = inGraph->GetErrorY( ii );

	sumy += y/(sigma*sigma);
	sumSigmaSquared += 1/(sigma*sigma);
    }

    return retVal;
}
