/*
 *  Time-based histogram maker.
 *  Based on the mipHistogram_maker.C code.  Modified by Wayne Witzke, for the
 *  University of Kentucky, Department of Physics & Astronomy.
 */

#include <iostream>
#include <fstream>
#include <set>
#include <climits>
using namespace std;

void time_eta_histo_maker(const char *file_list="/star/u/wwitzke/workspace/mipcode/2008_file_list",
			 //const char *file_list="/star/data07/EMC/mattheww/offline_tower_calibration/2006/ppProd/root/calib/BE6089404D8E758EE247FC81E96086D9_*.root",
			 //const char* file_list="/star/data07/EMC/mattheww/offline_tower_calibration/2006/ppProd/root/mip/*mip.root",
			 const char* skimfile="/star/institutions/uky/wwitzke/mip/mipskimfile.root") 

{
    gROOT->Macro("LoadLogger.C");
    gROOT->Macro("$STAR/StRoot/StMuDSTMaker/COMMON/macros/loadSharedLibraries.C");
    gSystem->Load("StTpcDb");
    gSystem->Load("StDaqLib");
    gSystem->Load("StDetectorDbMaker");
    gSystem->Load("St_db_Maker");
    gSystem->Load("StDbUtilities");
    gSystem->Load("StEmcRawMaker");
    gSystem->Load("StMcEvent");
    gSystem->Load("StMcEventMaker");//***
    gSystem->Load("StEmcSimulatorMaker");//***
    gSystem->Load("StEmcADCtoEMaker");
    gSystem->Load("StEpcMaker");
    gSystem->Load("StDbBroker");
    gSystem->Load("StEEmcUtil");
    gSystem->Load("StAssociationMaker");
    gSystem->Load("StEmcTriggerMaker");
    //gSystem->Load("StEmcDecoder");

    gSystem->Load("StEmcOfflineCalibrationMaker");

    const int ntowers=4800;
    const int neta = 40;
    const bool lookForSwaps=false;

    cout<<"input filelist:  "<<file_list<<endl;
    cout<<"histogram file:  "<<skimfile<<endl;

    //chain all input files together
    //char file[3000]=file_list;
    char file[60000];
    TChain* calib_tree = new TChain("calibTree");
    //calib_tree->Add(file);
    ifstream filelist(file_list);
    TFile *test_file;
    while(1)
    {
	filelist >> file;
	cout<<" File="<<file<<endl;
	if(!filelist.good()) break;
	calib_tree->Add(file);
    }

    /*
    St_db_Maker myDBMaker(
	"StarDb", "MySQL:StarDb", "MySQL:StarDb", "$STAR/StarDb"
    );
    */

    StEmcOfflineCalibrationEvent* myEvent = new StEmcOfflineCalibrationEvent();
    calib_tree->SetBranchAddress("event_branch",&myEvent);
    StEmcOfflineCalibrationTrack* mip;
    StEmcGeom etaFinder("bemc");
    //StEmcDecoder etaFinder2;

    //	This is just so that we have a complete set of eta values for each
    //	tower id.
    /*
    ofstream etaFile( "etaBinsByTower.txt" );
    for ( int ii = 0; ii < 4801; ++ii )
    {
	float currentEta;
	if ( etaFinder.getEta(ii, currentEta) )
	{
	    etaFile << "No eta for tower number " << ii << endl;
	}
	else
	{
	    etaFile << "Tower " << ii << ": " << currentEta << endl;
	}
    }

    etaFile.close();
    */

    //create the 4800 mip histograms
    /*
    char name[100];
    TH1* mipHisto[ntowers];
    for(int k=0; k<ntowers; k++)
    {
	sprintf(name,"mipHisto_%i",k+1);
	mipHisto[k] = new TH2D(name,name,250,-50.5,199.5);
    }
    */

    //	It turns out we have to preprocess all entries so that we know the
    //	range of run numbers.
    unsigned int minRun = numeric_limits<unsigned int>::max();
    unsigned int maxRun = 0;
    unsigned int nentries = calib_tree->GetEntries();
    //set<int> runs;
    cout << "Preprocessing to find run numbers. . ." << endl;
    cout << "(total entries " << nentries << ")" << endl;
    set<int> runs;
    for(unsigned int ii=0; ii<nentries; ++ii)
    {
	if(ii%100000 == 0)
	{
	    cout << "Pre-processing "<< ii <<" of " << nentries << endl;
	}

	calib_tree->GetEntry(ii);

	unsigned int curRun = myEvent->fill;
	runs->insert(myEvent->run);

	if ( curRun < minRun )
	    minRun = curRun;

	if ( curRun > maxRun )
	    maxRun = curRun;

	//runs.insert(curRun);
    }
    set<int>::iterator runIterator;
    ofstream runNumbers("/star/institutions/uky/wwitzke/mip/runNumbers");
    for ( runIterator = runs.begin(); runIterator != runs.end(); ++runIterator )
    {
	runNumbers << *runIterator << endl;
    }
    close runNumbers;
    cout << "Minimum fill number: " << minRun << endl;
    cout << "Maximum fill number: " << maxRun << endl;

    return;

    /*
    char * name = "time_mipHisto";
    //	I want to create a two-dimensionsal bin here, so that I can see the
    //	variations of bins over time.  
    TH2D * mipHisto = new TH2D(name,name,250,-50.5,199.5,runs.size(),minRun,maxRun);
    //mipHisto->SetBit(TH1::kCanRebin);
    */
    //	We are going to create a separate histogram for each run number.
    char name[100];
    TH1*** mipHisto = new TH1**[maxRun-minRun + 1];
    for(unsigned int ii = 0; ii < maxRun-minRun+1; ++ii)
    {
	mipHisto[ii] = new TH1*[neta];
	for (unsigned int jj = 0; jj < neta; ++jj)
	{
	    sprintf(name,"mipHisto_run_%i_eta_%i",minRun+ii,jj+1);
	    mipHisto[ii][jj] = new TH1D(name,name,250,-50.5,199.5);
	}
    }

    //keep track of all hit towers and exclude any with >1 track/tower
    set<int> track_towers;
    set<int> excluded_towers;

    for(unsigned int ii=0; ii<nentries; ++ii)
    {
	unsigned int runNum = myEvent->fill;

	if(ii%100000 == 0)
	{
	    cout << "processing " << ii << " of " << nentries << endl;
	    cout << "Processing fill number: " << runNum << endl;
	}

	//  We can also grab the run number here.  We'll be using this to
	//  speicfy the y-bin to fill later on.  This is more efficient than
	//  dereferencing the run number every time we do a fill.

	track_towers.clear();
	excluded_towers.clear();

	calib_tree->GetEntry(ii);

	//we're only working with events from vertexIndex==0
	if(TMath::Abs(myEvent->vz[0]) > 30.)	continue;

	//do a quick loop over tracks to get excluded towers
	for(unsigned int jj=0; jj<myEvent->tracks->GetEntries(); ++jj)
	{
	    mip = (StEmcOfflineCalibrationTrack*)myEvent->tracks->At(jj);
	    int id = mip->tower_id[0];

	    if(track_towers.find(id) != track_towers.end()){
	    excluded_towers.insert(id);
	    }
	    else
	    {
		track_towers.insert(id);
	    }
	}   

	//select on runnumbers to look for stability
	//if(myEvent->run < 7135000) continue;

	for(unsigned int jj=0; jj<myEvent->tracks->GetEntries(); jj++)
	{
	    mip = (StEmcOfflineCalibrationTrack*)myEvent->tracks->At(jj);

	    //subtract pedestal
	    double pedsub = mip->tower_adc[0] - mip->tower_pedestal[0];

	    if(mip->p < 1.)
	    {
		continue;
	    }
	    if(mip->tower_status[0] != 1)
	    {
		continue;
	    }
	    if(mip->highest_neighbor > 2.)
	    {
		continue;
	    }
	    if(TMath::Abs(pedsub) < 1.5*mip->tower_pedestal_rms[0])
	    {
		continue;
	    }
	    if(mip->tower_id[0] != mip->tower_id_exit)
	    {
		continue;
	    }
	    if(mip->vertexIndex > 0)
	    {
		continue;
	    }
	    if(excluded_towers.find(mip->tower_id[0]) != excluded_towers.end())
	    {
		continue;
	    }

	    //int index = mip->tower_id[0];
	    //	This is how it *should* be working, but unfortunately we're
	    //	using old code.
	    //int curEta = (int) mip->eta*40;
	    //	So, instead, to get eta, we use this:
	    int curModule_I;
	    int curEta_I;
	    int curSub_I;
	    float curEta;
	    if ( etaFinder.getEta( mip->tower_id[0], curEta ) )
	    {
		cout << "Eta error. . ?" << endl;
		cout.flush();
		continue;
	    }
	    //cout << "Eta is: " << curEta << endl;
	    //cout.flush();
	    if (
		etaFinder.getBin(
		    mip->tower_id[0], curModule_I, curEta_I, curSub_I
		)
	    )
	    {
		cout << "Eta bin error. . ?" << endl;
		cout.flush();
		continue;
	    }
	    //cout << "Integer eta is: " << curEta_I << endl;
	    //cout.flush();

	    if ( curEta_I < 0 || curEta_I > 39 )
	    {
		cout << curEta_I << "!" << endl;
		cout.flush();
	    }

	    mipHisto[runNum-minRun][curEta_I]->Fill(pedsub);
	}
    }

    TFile* output_file = new TFile(skimfile,"RECREATE");
    for ( int kk=0; kk < maxRun-minRun+1; ++kk )
	for ( int ll = 0; ll < neta; ++ll )
	    mipHisto[kk][ll]->Write();
    //mipHisto->Write();
    output_file->Close();
    cout << "Done writing histograms" << endl;
    cout.flush();
}
