/*
 *  Fill-based histogram maker.
 *  Based on the mip_histogram_maker.C code.  Modified by Wayne Witzke, for the
 *  University of Kentucky, Department of Physics & Astronomy.
 */

#include <iostream>
#include <fstream>
#include <set>
#include <climits>
using namespace std;

void fill_based_histo_maker(const char *file_list="/star/u/wwitzke/workspace/mipcode/next_file_list",
			 //const char *file_list="/star/data07/EMC/mattheww/offline_tower_calibration/2006/ppProd/root/calib/BE6089404D8E758EE247FC81E96086D9_*.root",
			 //const char* file_list="/star/data07/EMC/mattheww/offline_tower_calibration/2006/ppProd/root/mip/*mip.root",
			 const char* skimfile="/star/institutions/uky/wwitzke/mip/mipskimfile.root") 

{
    gROOT->Macro("LoadLogger.C");
    gROOT->Macro("$STAR/StRoot/StMuDSTMaker/COMMON/macros/loadSharedLibraries.C");
    gSystem->Load("StTpcDb");
    gSystem->Load("StDaqLib");
    gSystem->Load("StDetectorDbMaker");
    gSystem->Load("St_db_Maker");
    gSystem->Load("StDbUtilities");
    gSystem->Load("StEmcRawMaker");
    gSystem->Load("StMcEvent");
    gSystem->Load("StMcEventMaker");//***
    gSystem->Load("StEmcSimulatorMaker");//***
    gSystem->Load("StEmcADCtoEMaker");
    gSystem->Load("StEpcMaker");
    gSystem->Load("StDbBroker");
    gSystem->Load("StEEmcUtil");
    gSystem->Load("StAssociationMaker");
    gSystem->Load("StEmcTriggerMaker");

    gSystem->Load("StEmcOfflineCalibrationMaker");

    const int ntowers=4800;
    const bool lookForSwaps=false;

    cout<<"input filelist:  "<<file_list<<endl;
    cout<<"histogram file:  "<<skimfile<<endl;

    //chain all input files together
    //char file[3000]=file_list;
    char file[60000];
    TChain* calib_tree = new TChain("calibTree");
    //calib_tree->Add(file);
    ifstream filelist(file_list);
    TFile *test_file;
    while(1)
    {
	filelist >> file;
	cout<<" File="<<file<<endl;
	if(!filelist.good()) break;
	calib_tree->Add(file);
    }

    StEmcOfflineCalibrationEvent* myEvent = new StEmcOfflineCalibrationEvent();
    calib_tree->SetBranchAddress("event_branch",&myEvent);
    StEmcOfflineCalibrationTrack* mip;

    //create the 4800 mip histograms
    /*
    char name[100];
    TH1* mip_histo[ntowers];
    for(int k=0; k<ntowers; k++)
    {
	sprintf(name,"mip_histo_%i",k+1);
	mip_histo[k] = new TH2D(name,name,250,-50.5,199.5);
    }
    */

    //	It turns out we have to preprocess all entries so that we know the
    //	range of run numbers.
    unsigned int minFill = numeric_limits<unsigned int>::max();
    unsigned int maxFill = 0;
    unsigned int nentries = calib_tree->GetEntries();
    //set<int> fills;
    cout << "Preprocessing to find fill numbers. . ." << endl;
    for(unsigned int i=0; i<nentries; i++)
    {
	if(i%100000 == 0)
	{
	    cout << "Pre-processing "<< i <<" of " << nentries << endl;
	}

	calib_tree->GetEntry(i);

	unsigned int curFill = myEvent->fill;
	if ( curFill < minFill )
	    minFill = curFill;

	if ( curFill > maxFill )
	    maxFill = curFill;

	//fills.insert(curFill);
    }
    cout << "Minimum fill number: " << minFill << endl;
    cout << "Maximum fill number: " << maxFill << endl;

    /*
    char * name = "time_mip_histo";
    //	I want to create a two-dimensionsal bin here, so that I can see the
    //	variations of bins over time.  
    TH2D * mip_histo = new TH2D(name,name,250,-50.5,199.5,fills.size(),minRun,maxRun);
    //mip_histo->SetBit(TH1::kCanRebin);
    */
    //	We are going to create a separate histogram for each run number.
    char name[100];
    TH1 *** mip_histo;
    mip_histo = new TH1**[ntowers];
    //TH1 * mip_histo[ntowers][];
    for ( int ii = 0; ii < ntowers; ++ii)
    {
	mip_histo[ii] = new TH1*[maxFill-minFill+1];
	//  This really shouldn't be necessary. . .
	for ( int jj = 0; jj < (maxFill-minFill+1); ++jj )
	{
	    mip_histo[ii][jj] = 0;
	}
    }
    //	We'll create the mip_histo[][] dynamically later on.
    /*
    for(int ii=0; ii<(maxFill-minFill); ii++)
    {
	for (int jj=0; jj<ntowers; jj++)
	{
	    sprintf(name,"mip_histo_fill_%i_tower_%i",ii+1,jj+1);
	    mip_histo[ii][jj] = new TH1D(name,name,250,-50.5,199.5);
	}
    }
    */

    //keep track of all hit towers and exclude any with >1 track/tower
    set<int> track_towers;
    set<int> excluded_towers;

    for(unsigned int i=0; i<nentries; i++)
    {
	track_towers.clear();
	excluded_towers.clear();

	calib_tree->GetEntry(i);

	unsigned int fillNum = myEvent->fill;

	if(i%100000 == 0)
	{
	    cout<<"processing "<<i<<" of "<<nentries<<endl;
	    //cout << "Processing fill number: " << fillNum << endl;
	}

	//we're only working with events from vertexIndex==0
	if(TMath::Abs(myEvent->vz[0]) > 30.)	continue;

	//do a quick loop over tracks to get excluded towers
	for(int j=0; j<myEvent->tracks->GetEntries(); j++)
	{
	    mip = (StEmcOfflineCalibrationTrack*)myEvent->tracks->At(j);
	    int id = mip->tower_id[0];

	    if(track_towers.find(id) != track_towers.end()){
	    excluded_towers.insert(id);
	    }
	    else
	    {
		track_towers.insert(id);
	    }
	}   

	//select on runnumbers to look for stability
	//if(myEvent->run < 7135000) continue;

	for(int j=0; j<myEvent->tracks->GetEntries(); j++)
	{
	    mip = (StEmcOfflineCalibrationTrack*)myEvent->tracks->At(j);

	    //subtract pedestal
	    double pedsub = mip->tower_adc[0] - mip->tower_pedestal[0];

	    if(mip->p < 1.) continue;
	    if(mip->tower_status[0] != 1) continue;
	    if(mip->highest_neighbor > 2.) continue;
	    if(TMath::Abs(pedsub) < 1.5*mip->tower_pedestal_rms[0]) continue;
	    if(mip->tower_id[0] != mip->tower_id_exit) continue;
	    if(mip->vertexIndex > 0) continue;
	    if(excluded_towers.find(mip->tower_id[0]) != excluded_towers.end())
		continue;

	    int index = mip->tower_id[0];
	    if ( !(mip_histo[index-1][fillNum-minFill]) )
	    {
		sprintf(name,"mip_histo_tower_%i_fill_%i",index,fillNum);
		mip_histo[index-1][fillNum-minFill] =
		    new TH1D(name,name,250,-50.5,199.5);
	    }
	    mip_histo[index-1][fillNum-minFill]->Fill(pedsub);
	}
    }

    TFile* output_file = new TFile(skimfile,"RECREATE");
    for(int ii=0; ii < ntowers; ++ii)
    {
	for ( int jj=0; jj < (maxFill-minFill+1); ++jj)
	{
	    if ( mip_histo[ii][jj] )
	    {
		mip_histo[ii][jj]->Write();
	    }
	}
    }
    //mip_histo->Write();
    output_file->Close();
}
